#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <windows.h>
// Include important header files needed to interface with Outpost 2
#include <Outpost2DLL.h>
// Include header files to make it easier to build levels
#include <OP2Helper.h>

// These are needed for the mission briefing
#include <richedit.h>
#include "resource.h"
#include "odasl\odasl.h"

// Disable warning 4244, otherwise we get lots of warnings about double -> int conversions
#pragma warning(disable:4244)

// Forward declare all functions
extern void SetupObjects();
void SetupAIMines();
void SetupAIFactories();
void SetupAIDefense();

char MapName[]			= "eden03.map";							// The .map file used for this level
char LevelDesc[]		= "Plymouth Cold War";					// Description appearing in the game list box
char TechtreeName[]		= "MULTITEK.TXT";						// File to use for the tech tree
SDescBlock DescBlock	= { Colony, 2, 12, 0 }; 				// Important level details

// We must save all triggers and groups. The game does not call InitProc when the game is loaded.
// This way, we can recreate all the triggers/groups.
struct
{
    Trigger waitTrig;
    Trigger killTrig;
    Trigger aiStateTrig;
    Trigger hasSFTrig;
    Trigger failTrig;
    Trigger disasterTrig;
    MiningGroup aiMineGrp[3]; // AI mining
    BuildingGroup aiBuildGrp; // AI SF / VF
    FightGroup aiDefGrp[7]; // AI defense
    FightGroup aiMassGrp; // Group used to mass up the AI attack units
    FightGroup aiAttackGrp; // Group used to attack the player base
    int aiCount; // this counter is incremented by AIStateChange each run
    bool usePanthers;
    bool useTigers;
    double diffMultiplier; // difficulty multiplier used to change amounts based on diff. level
} saveData;

// List of songs to play
SongIds songs[] = {
    songEden11,
    songEP41,
    songEP43,
    songEP52,
    songEP51,
    songEP62,
    songEP61,
    songEP63
};

// These deal with the briefing screen
HINSTANCE hInst;
LRESULT CALLBACK DialogProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

BOOL APIENTRY DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	if (fdwReason == DLL_PROCESS_ATTACH)
	{
		DisableThreadLibraryCalls(hinstDLL);
		hInst = hinstDLL; // Must save the DLL's HINSTANCE
	}
    return TRUE;
}

int InitProc()
{
    // OP2 turns off the skinning before the game loads.
    // We have to re-enable it (otherwise the briefing box will look ugly).
    wplEnable();
    // Show the dialog box, using the shell window as the parent
	HWND shellWnd = FindWindow("Outpost2Shell", NULL);
	DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_MISSIONINFO), shellWnd, (DLGPROC)DialogProc, NULL);
    // Turn off the skinning again (probably not needed but a good idea to do it anyway)
    wplDisable();

	TethysGame::ForceMoraleGood(0);
	TethysGame::ForceMoraleGood(1);

	Player[0].GoEden();
    saveData.diffMultiplier = 0.7;
	Player[0].MarkResearchComplete(techResearchTrainingPrograms);
	Player[0].MarkResearchComplete(techOffspringEnhancement);
	Player[0].MarkResearchComplete(techCyberneticTeleoperation);

    if (Player[0].Difficulty() < 2)
    {
        saveData.diffMultiplier = 1;
        Player[0].MarkResearchComplete(techLargeScaleOpticalResonators);
        Player[0].MarkResearchComplete(techHighTemperatureSuperconductivity);
        Player[0].MarkResearchComplete(techMobileWeaponsPlatform);
        Player[0].MarkResearchComplete(techMetallogeny);
        if (Player[0].Difficulty() < 1)
        {
            saveData.diffMultiplier = 1.3;
            Player[0].MarkResearchComplete(techExplosiveCharges);
            Player[0].MarkResearchComplete(techScoutClassDriveTrainRefit);
        }
    }
    Player[0].SetOre(2500 * saveData.diffMultiplier);
	Player[0].SetWorkers(15 * saveData.diffMultiplier);
	Player[0].SetScientists(7 * saveData.diffMultiplier);
	Player[0].SetFoodStored(1200 * saveData.diffMultiplier);

    Player[1].GoPlymouth();
    Player[1].GoAI();
	Player[1].SetOre(2000);
	Player[1].SetTechLevel(12);

	// Setup human player ore
    TethysGame::CreateBeacon(mapMiningBeacon, 33+31, 32-1, 0, 2, 0);
	TethysGame::CreateBeacon(mapMiningBeacon, 59+31, 27-1, 0, Player[0].Difficulty(), -1);
    TethysGame::CreateBeacon(mapMiningBeacon, 20+31, 56-1, 0, -1, -1);

    TethysGame::CreateBeacon(mapMiningBeacon, 31+31, 11-1, 1, Player[0].Difficulty(), -1);
    TethysGame::CreateBeacon(mapMiningBeacon, 83+31, 45-1, 1, -1, -1);

	SetupObjects(); // create OP2Mapper units

	SetupAIFactories();
    SetupAIMines();
    SetupAIDefense();

    // Set reinforce info
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiDefGrp[0], 0);
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiDefGrp[1], 5000); // Very high priority; it's the lab
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiDefGrp[2], 5000); // Also high; the CC
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiDefGrp[3], 5000); // Factories
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiDefGrp[4], 0);
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiDefGrp[5], 0);
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiDefGrp[6], 0);

	// Turn on vehicle lights
	PlayerVehicleEnum vehEnum(0);
	Unit veh;
	while (vehEnum.GetNext(veh))
        veh.DoSetLights(1);

    vehEnum = PlayerVehicleEnum(1);
	while (vehEnum.GetNext(veh))
        veh.DoSetLights(1);

	// Map lighting settings
	TethysGame::SetDaylightMoves(1);
	TethysGame::SetDaylightEverywhere(0);

	// Setup trigger to send reinforcements once the player begins to accumulate ore
	saveData.waitTrig = CreateResourceTrigger(1, 1, resCommonOre, Player[0].Ore(), 0, cmpGreater, "InitialReinforce");
	CreateVictoryCondition(1, 0, saveData.waitTrig, "Establish a mining operation.");

    saveData.killTrig = CreateCountTrigger(1, 0, 1, mapAdvancedLab, mapAny, 0, cmpEqual, "NoResponseToTrigger");
    CreateVictoryCondition(1, 0, saveData.killTrig, "Destroy the Plymouth Advanced Lab.");

    saveData.failTrig = CreateOperationalTrigger(1, 1, 0, mapCommandCenter, 0, cmpEqual, "NoResponseToTrigger");
    CreateFailureCondition(1, 0, saveData.failTrig, "");

    // Start the offensive AI groups going
    saveData.aiMassGrp = CreateFightGroup(Player[1]);
    saveData.aiMassGrp.SetRect(MAP_RECT(70+31, 15-1, 95+31, 50-1));
    saveData.aiBuildGrp.RecordVehReinforceGroup(saveData.aiMassGrp, 3 - (saveData.diffMultiplier * 1.5));
    saveData.aiMassGrp.SetTargCount(mapLynx, mapMicrowave, 1); // Set a modest one to start

    saveData.aiAttackGrp = CreateFightGroup(Player[1]);
    saveData.aiAttackGrp.SetRect(MAP_RECT(20+31, 10-1, 80+31, 60-1));

    // Start the periodic changes slowly.. once the player has a mine, start going faster ;)
    // and do some other processing, like teching up, etc
    saveData.aiCount = 0;
    saveData.usePanthers = false;
    saveData.useTigers = false;
    saveData.aiStateTrig = CreateTimeTrigger(1, 0, 1200 * saveData.diffMultiplier, 1800 * saveData.diffMultiplier, "AIStateChange");

    // Wait for the user to have capability to build weapons before increasing the chances of attack
    saveData.hasSFTrig = CreateOperationalTrigger(1, 1, 0, mapStructureFactory, 1, cmpGreaterEqual, "NoResponseToTrigger");

	// Set up some disasters! ;)
	saveData.disasterTrig = CreateTimeTrigger(1, 0, 1500 * saveData.diffMultiplier, 2500 * saveData.diffMultiplier, "Disasters");

    // Give some faster music
    TethysGame::SetMusicPlayList(6, 3, songs);
	return 1; // return 1 if OK; 0 on failure
}

void AIProc()
{
}

void __cdecl GetSaveRegions(struct BufferDesc &bufDesc)
{
    // Note: this has been changed. The count must be persisted in the saved game file,
    // so that the AI is in the right state when the game is reloaded.

	bufDesc.bufferStart = &saveData;     // Pointer to a buffer that needs to be saved
	bufDesc.length = sizeof(saveData);	// sizeof(buffer)
}

int StatusProc()
{
	return 0; // must return 0
}

SCRIPT_API void NoResponseToTrigger()
{
}

SCRIPT_API void __cdecl AIStateChange()
{
	map_id curType;
    // Increment a counter
    saveData.aiCount++;

    // Should AI attack?
    if (TethysGame::GetRand(saveData.hasSFTrig.HasFired(0) ? 2 : 3) == 0)
    {
        saveData.aiAttackGrp.TakeAllUnits(saveData.aiMassGrp);
        saveData.aiAttackGrp.DoAttackEnemy();
    }

    // Adjust what types of units to make based on the player's military strength
    if (Player[0].GetTotalPlayerStrength() >= 20 * saveData.diffMultiplier)
    {
    	saveData.useTigers = true;
    	saveData.aiCount = 0;
    }
    else if (Player[0].GetTotalPlayerStrength() >= 10 * saveData.diffMultiplier)
    {
    	saveData.usePanthers = true;
    	saveData.aiCount = 0;
    }

    // Decide what type to create
    curType = (saveData.useTigers ? mapTiger : (saveData.usePanthers ? mapPanther : mapLynx));

    // Check the count and increase the AI strength as needed
    if (saveData.aiCount == 10 * saveData.diffMultiplier)
    {
        // Starflare time
        saveData.aiMassGrp.SetTargCount(curType, mapStarflare, 3 - (saveData.diffMultiplier * 1.5));
    }
    else if (saveData.aiCount == 20 * saveData.diffMultiplier)
    {
        // Stickyfoam time
        saveData.aiMassGrp.SetTargCount(curType, mapStickyfoam, 3 - (saveData.diffMultiplier * 1.5));
    }
    else if (saveData.aiCount == 30 * saveData.diffMultiplier)
    {
        // EMP time
        saveData.aiMassGrp.SetTargCount(curType, mapEMP, 3 - (saveData.diffMultiplier * 1.5));
    }
    else if (saveData.aiCount == 45 * saveData.diffMultiplier)
    {
        // RPG time
        saveData.aiMassGrp.SetTargCount(curType, mapRPG, 3 - (saveData.diffMultiplier * 1.5));
    }
    else if (saveData.aiCount == 70 * saveData.diffMultiplier)
    {
        // ESG time
        saveData.aiMassGrp.SetTargCount(curType, mapESG, 3 - (saveData.diffMultiplier * 1.5));
    }
    else if (saveData.aiCount == 75 * saveData.diffMultiplier)
    {
        // Nova time
        saveData.aiMassGrp.SetTargCount(curType, mapSupernova, 3 - (saveData.diffMultiplier * 1.5));
    }
}

SCRIPT_API void __cdecl InitialReinforce()
{
	// Give the player some stuff to expand their base with
	Unit u;
	TethysGame::CreateUnit(u, mapConVec, LOCATION(0+31, 44-1), 0, mapStructureFactory, 0);
	u.DoSetLights(1);
	u.DoMove(LOCATION(34+31, 34-1));

    if (Player[0].Difficulty() < 2)
    {
        TethysGame::CreateUnit(u, mapEarthworker, LOCATION(0+31, 45-1), 0, mapNone, 0);
        u.DoSetLights(1);
        u.DoMove(LOCATION(36+31, 34-1));
        if (Player[0].Difficulty() < 1)
        {
            TethysGame::CreateUnit(u, mapConVec, LOCATION(0+31, 46-1), 0, mapStandardLab, 0);
            u.DoSetLights(1);
            u.DoMove(LOCATION(35+31, 34-1));
        }
    }
	if (Player[0].Difficulty() > 0) // unless the player has it on easy mode let the colonists revolt ;P
        TethysGame::FreeMoraleLevel(0);

    // Start the attacks faster
    saveData.aiStateTrig.Destroy();
    saveData.aiStateTrig = CreateTimeTrigger(1, 0, 800 * saveData.diffMultiplier, 1400 * saveData.diffMultiplier, "AIStateChange");

	TethysGame::AddMessage(u, "Reinforcements have arrived", 0, sndSavnt205);
}

SCRIPT_API void __cdecl Disasters()
{
	// Set a disaster. This uses a weighted random. Picking a random number 0 to 100:
	// 0-19 = no action
	// 20-69 = meteor
	// 70-84 = quake
	// 85-94 = electrical storm
	// 95-100 = vortex
	// Note: the X coordinate in the GetRand calls is only 112, because we wouldn't want
	// a vortex to take out the lab or any other AI vitals for us. ;)

	int randNum = TethysGame::GetRand(100);

	if (20*saveData.diffMultiplier <= randNum && randNum < 70*saveData.diffMultiplier)
	{
		// Meteor
		TethysGame::SetMeteor(TethysGame::GetRand(112)+31, TethysGame::GetRand(127)-1, TethysGame::GetRand(5-5*saveData.diffMultiplier)+1);
	}
	else if (70*saveData.diffMultiplier <= randNum && randNum < 85*saveData.diffMultiplier)
	{
		// Quake
		TethysGame::SetEarthquake(TethysGame::GetRand(112)+31, TethysGame::GetRand(127)-1, TethysGame::GetRand(3-3*saveData.diffMultiplier)+1);
	}
	else if (85*saveData.diffMultiplier <= randNum && randNum < 95*saveData.diffMultiplier)
	{
		// Lightning
		TethysGame::SetLightning(TethysGame::GetRand(112)+31, TethysGame::GetRand(127)-1, TethysGame::GetRand(20-10*saveData.diffMultiplier)+5, TethysGame::GetRand(112)+31, TethysGame::GetRand(127)-1);
	}
	else if (95*saveData.diffMultiplier <= randNum)
	{
		// Tornado
		TethysGame::SetTornado(TethysGame::GetRand(112)+31, TethysGame::GetRand(127)-1, TethysGame::GetRand(20-10*saveData.diffMultiplier)+5, TethysGame::GetRand(112)+31, TethysGame::GetRand(127)-1, 1);
	}
}

void SetupAIMines()
{
    // Setup AI mining units and routes
	Unit mine, smelter, truck;
    // Common Group 1
    saveData.aiMineGrp[0] = CreateMiningGroup(Player[1]);
    TethysGame::CreateUnit(mine, mapCommonOreMine, LOCATION(92+31, 53-1), 1, mapNone, 0);
    saveData.aiMineGrp[0].TakeUnit(mine);
    TethysGame::CreateUnit(smelter, mapCommonOreSmelter, LOCATION(107+31, 43-1), 1, mapNone, 0);
    saveData.aiMineGrp[0].TakeUnit(smelter);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(102+31, 38-1), 1, mapNone, 0);
    saveData.aiMineGrp[0].TakeUnit(truck);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(102+31, 40-1), 1, mapNone, 0);
    saveData.aiMineGrp[0].TakeUnit(truck);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(101+31, 39-1), 1, mapNone, 0);
    saveData.aiMineGrp[0].TakeUnit(truck);
    saveData.aiMineGrp[0].Setup(mine, smelter, MAP_RECT(107-3+31, 43-2-1, 107+2+31, 43+2-1));

    // Common Group 2
    saveData.aiMineGrp[1] = CreateMiningGroup(Player[1]);
    TethysGame::CreateUnit(mine, mapCommonOreMine, LOCATION(87+31, 31-1), 1, mapNone, 0);
    saveData.aiMineGrp[1].TakeUnit(mine);
    TethysGame::CreateUnit(smelter, mapCommonOreSmelter, LOCATION(107+31, 39-1), 1, mapNone, 0);
    saveData.aiMineGrp[1].TakeUnit(smelter);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(99+31, 38-1), 1, mapNone, 0);
    saveData.aiMineGrp[1].TakeUnit(truck);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(98+31, 39-1), 1, mapNone, 0);
    saveData.aiMineGrp[1].TakeUnit(truck);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(99+31, 40-1), 1, mapNone, 0);
    saveData.aiMineGrp[1].TakeUnit(truck);
    saveData.aiMineGrp[1].Setup(mine, smelter, MAP_RECT(107-3+31, 39-2-1, 107+2+31, 39+2-1));

	// Rare Group - north smelter
	saveData.aiMineGrp[2] = CreateMiningGroup(Player[1]);
	TethysGame::CreateUnit(mine, mapCommonOreMine, LOCATION(99+31, 19-1), 1, mapNone, 0);
	saveData.aiMineGrp[2].TakeUnit(mine);
	TethysGame::CreateUnit(smelter, mapRareOreSmelter, LOCATION(110+31, 26-1), 1, mapNone, 0);
	saveData.aiMineGrp[2].TakeUnit(smelter);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(104+31, 21-1), 1, mapNone, 0);
    saveData.aiMineGrp[2].TakeUnit(truck);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(102+31, 22-1), 1, mapNone, 0);
    saveData.aiMineGrp[2].TakeUnit(truck);
    TethysGame::CreateUnit(truck, mapCargoTruck, LOCATION(106+31, 22-1), 1, mapNone, 0);
    saveData.aiMineGrp[2].TakeUnit(truck);
    saveData.aiMineGrp[2].Setup(mine, smelter, MAP_RECT(110-3+31, 26-2-1, 110+2+31, 26+2-1));
}

void SetupAIFactories()
{
	// Setup AI factories, Convecs, Earthworker, building group
	Unit sf, vf, convec, ewc;
	saveData.aiBuildGrp = CreateBuildingGroup(Player[1]);

    TethysGame::CreateUnit(convec, mapConVec, LOCATION(100+31, 46-1), 1, mapNone, 0);
    saveData.aiBuildGrp.TakeUnit(convec);
    TethysGame::CreateUnit(convec, mapConVec, LOCATION(102+31, 46-1), 1, mapNone, 0);
    saveData.aiBuildGrp.TakeUnit(convec);
    TethysGame::CreateUnit(convec, mapConVec, LOCATION(104+31, 46-1), 1, mapNone, 0);
    saveData.aiBuildGrp.TakeUnit(convec);
    TethysGame::CreateUnit(ewc, mapEarthworker, LOCATION(106+31, 46-1), 1, mapNone, 0);
	saveData.aiBuildGrp.TakeUnit(ewc);

    TethysGame::CreateUnit(sf, mapStructureFactory, LOCATION(102+31, 43-1), 1, mapNone, 0);
    saveData.aiBuildGrp.TakeUnit(sf);

    TethysGame::CreateUnit(vf, mapVehicleFactory, LOCATION(118+31, 49-1), 1, mapNone, 0);
    saveData.aiBuildGrp.TakeUnit(vf);
    TethysGame::CreateUnit(vf, mapVehicleFactory, LOCATION(115+31, 45-1), 1, mapNone, 0);
    saveData.aiBuildGrp.TakeUnit(vf);

    saveData.aiBuildGrp.SetRect(MAP_RECT(98+31, 45-1, 110+31, 47-1));

    // Record everything so the AI will rebuild it
    saveData.aiBuildGrp.RecordTube(LOCATION(124+31, 55-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(124+31, 54-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(124+31, 53-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(123+31, 53-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(122+31, 53-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(121+31, 53-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(121+31, 52-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(121+31, 51-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(121+31, 50-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(121+31, 49-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(112+31, 46-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(111+31, 46-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 46-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 45-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 44-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 43-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 42-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 41-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 40-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 39-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 38-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 37-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 36-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 35-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 32-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(111+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(112+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(113+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(114+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(115+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(116+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(117+31, 33-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(117+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(118+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(119+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(120+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(121+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(122+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(123+31, 34-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(125+31, 36-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(125+31, 37-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(118+31, 35-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 29-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 30-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(110+31, 31-1));
	saveData.aiBuildGrp.RecordTube(LOCATION(115+31, 48-1));
	saveData.aiBuildGrp.RecordBuilding(LOCATION(87+31, 59-1), (map_id)58, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(91+31, 59-1), (map_id)58, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(87+31, 4-1), (map_id)58, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(83+31, 4-1), (map_id)58, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(91+31, 4-1), (map_id)58, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(124+31, 60-1), (map_id)28, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(124+31, 57-1), (map_id)39, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(125+31, 34-1), (map_id)49, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(125+31, 39-1), (map_id)50, (map_id)0);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(117+31, 31-1), (map_id)23, (map_id)71);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(118+31, 36-1), (map_id)23, (map_id)70);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(106+31, 27-1), (map_id)23, (map_id)71);
    saveData.aiBuildGrp.RecordBuilding(LOCATION(98+31, 43-1), (map_id)23, (map_id)64);
}

void SetupAIDefense()
{
	Unit lynx;

	// Group 1 - Defends north smelter
	saveData.aiDefGrp[0] = CreateFightGroup(Player[1]);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(94+31, 15-1), 1, mapMicrowave, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(97+31, 22-1), 1, mapMicrowave, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(93+31, 19-1), 1, mapMicrowave, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(90+31, 22-1), 1, mapRPG, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(96+31, 26-1), 1, mapRPG, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(102+31, 25-1), 1, mapMicrowave, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(104+31, 28-1), 1, mapMicrowave, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(108+31, 19-1), 1, mapMicrowave, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);
	TethysGame::CreateUnit(lynx, mapLynx, LOCATION(102+31, 16-1), 1, mapMicrowave, 0);
	saveData.aiDefGrp[0].TakeUnit(lynx);

	// Oddity in OP2: You must call DoGuardGroup before specifying the target group!
	saveData.aiDefGrp[0].DoGuardGroup();
	saveData.aiDefGrp[0].SetTargetGroup(saveData.aiMineGrp[2]);
	saveData.aiDefGrp[0].SetTargCount(mapLynx, mapMicrowave, 7);
	saveData.aiDefGrp[0].SetTargCount(mapLynx, mapRPG, 2);
    saveData.aiDefGrp[0].SetRect(MAP_RECT(90+31, 15-1, 112+31, 28-1));

    // Group 2 - Defends lab area
    saveData.aiDefGrp[1] = CreateFightGroup(Player[1]);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(107+31, 30-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[1].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(114+31, 31-1), 1, mapRPG, 0);
    saveData.aiDefGrp[1].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(112+31, 32-1), 1, mapRPG, 0);
    saveData.aiDefGrp[1].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(112+31, 34-1), 1, mapRPG, 0);
    saveData.aiDefGrp[1].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(112+31, 38-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[1].TakeUnit(lynx);
    saveData.aiDefGrp[1].AddGuardedRect(MAP_RECT(109+31, 31-1, 127+31, 33-1));
    saveData.aiDefGrp[1].DoGuardRect();
    saveData.aiDefGrp[1].SetTargCount(mapLynx, mapMicrowave, 2);
    saveData.aiDefGrp[1].SetTargCount(mapLynx, mapRPG, 3);
    saveData.aiDefGrp[1].SetRect(MAP_RECT(111+31, 29-1, 121+31, 37-1));

    // Group 3 - Defends CC
    saveData.aiDefGrp[2] = CreateFightGroup(Player[1]);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(124+31, 51-1), 1, mapRPG, 0);
    saveData.aiDefGrp[2].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(119+31, 52-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[2].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(127+31, 55-1), 1, mapRPG, 0);
    saveData.aiDefGrp[2].TakeUnit(lynx);
    saveData.aiDefGrp[2].AddGuardedRect(MAP_RECT(121+31, 54-1, 128+31, 64-1));
    saveData.aiDefGrp[2].DoGuardRect();
    saveData.aiDefGrp[2].SetTargCount(mapLynx, mapMicrowave, 1);
    saveData.aiDefGrp[2].SetTargCount(mapLynx, mapRPG, 2);
    saveData.aiDefGrp[2].SetRect(MAP_RECT(121+31, 54-1, 128+31, 64-1));

    // Group 4 - Defends Factories
    saveData.aiDefGrp[3] = CreateFightGroup(Player[1]);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(98+31, 36-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(95+31, 38-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(94+31, 41-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(94+31, 45-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(97+31, 46-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(102+31, 36-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(100+31, 49-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(94+31, 50-1), 1, mapRPG, 0);
    saveData.aiDefGrp[3].TakeUnit(lynx);
    saveData.aiDefGrp[3].AddGuardedRect(MAP_RECT(91+31, 34-1, 111+31, 50-1));
    saveData.aiDefGrp[3].DoGuardRect();
    saveData.aiDefGrp[3].SetTargCount(mapLynx, mapMicrowave, 7);
    saveData.aiDefGrp[3].SetTargCount(mapLynx, mapRPG, 1);
    saveData.aiDefGrp[3].SetRect(MAP_RECT(91+31, 34-1, 111+31, 50-1));

    // Group 5 - Defends mid mine
    saveData.aiDefGrp[4] = CreateFightGroup(Player[1]);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(85+31, 33-1), 1, mapRPG, 0);
    saveData.aiDefGrp[4].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(89+31, 28-1), 1, mapRPG, 0);
    saveData.aiDefGrp[4].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(82+31, 28-1), 1, mapRPG, 0);
    saveData.aiDefGrp[4].TakeUnit(lynx);
    saveData.aiDefGrp[4].AddGuardedRect(MAP_RECT(82+31, 27-1, 92+31, 36-1));
    saveData.aiDefGrp[4].DoGuardRect();
    saveData.aiDefGrp[4].SetTargCount(mapLynx, mapRPG, 3);
    saveData.aiDefGrp[4].SetRect(MAP_RECT(82+31, 27-1, 92+31, 36-1));

    // Group 6 - Defends lower mine and toks
    saveData.aiDefGrp[5] = CreateFightGroup(Player[1]);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(85+31, 51-1), 1, mapRPG, 0);
    saveData.aiDefGrp[5].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(88+31, 49-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[5].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(96+31, 55-1), 1, mapRPG, 0);
    saveData.aiDefGrp[5].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(88+31, 49-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[5].TakeUnit(lynx);
    saveData.aiDefGrp[5].AddGuardedRect(MAP_RECT(85+31, 48-1, 97+31, 61-1));
    saveData.aiDefGrp[5].DoGuardRect();
    saveData.aiDefGrp[5].SetTargCount(mapLynx, mapRPG, 2);
    saveData.aiDefGrp[5].SetTargCount(mapLynx, mapMicrowave, 2);
    saveData.aiDefGrp[5].SetRect(MAP_RECT(85+31, 48-1, 97+31, 64-1));

    // Group 7 - Defends north toks
    saveData.aiDefGrp[6] = CreateFightGroup(Player[1]);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(78+31, 3-1), 1, mapRPG, 0);
    saveData.aiDefGrp[6].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(86+31, 8-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[6].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(92+31, 8-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[6].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(77+31, 10-1), 1, mapMicrowave, 0);
    saveData.aiDefGrp[6].TakeUnit(lynx);
    TethysGame::CreateUnit(lynx, mapLynx, LOCATION(94+31, 2-1), 1, mapRPG, 0);
    saveData.aiDefGrp[6].TakeUnit(lynx);
    saveData.aiDefGrp[6].AddGuardedRect(MAP_RECT(75+31, 1-1, 98+31, 16-1));
    saveData.aiDefGrp[6].DoGuardRect();
    saveData.aiDefGrp[6].SetTargCount(mapLynx, mapRPG, 2);
    saveData.aiDefGrp[6].SetTargCount(mapLynx, mapMicrowave, 3);
    saveData.aiDefGrp[6].SetRect(MAP_RECT(75+31, 1-1, 98+31, 16-1));
}

LRESULT CALLBACK DialogProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    HRSRC txtRes = NULL;
    HWND richEdit = NULL;
	switch(msg)
	{
		case WM_INITDIALOG:
            // Make the BG black in the rich edit box
            richEdit = GetDlgItem(hWnd, IDC_TEXTAREA);
            SendMessage(richEdit, EM_SETBKGNDCOLOR, 0, 0);

            // Find the text resource
            txtRes = FindResource(hInst, MAKEINTRESOURCE(IDR_MISSIONTEXT), "TEXT");
            if (txtRes)
            {
                // Load it
                HGLOBAL resHdl = LoadResource(hInst, txtRes);
                if (resHdl)
                {
                    // Lock it and set the text
                    char *briefing = (char*)LockResource(resHdl);
                    if (briefing)
                    {
                        SetDlgItemText(hWnd, IDC_TEXTAREA, briefing);
                        return TRUE;
                    }
                }
            }
            SetDlgItemText(hWnd, IDC_TEXTAREA, "Error: Mission text could not be loaded.");
            return TRUE;
        case WM_COMMAND:
            if (wParam == IDOK)
            {
                EndDialog(hWnd, IDOK);
                return TRUE;
            }
            return FALSE;
	}
	return FALSE;
}

